// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_DSI_H
#define OB_DSI_H

namespace PortableServer
{

//
// Forward declarations
//
class DynamicImplementation;

} // End of namespace PortableServer


namespace CORBA
{

class ServerRequest;
typedef ServerRequest* ServerRequest_ptr;
typedef ServerRequest_ptr ServerRequestRef;

} // End of namespace CORBA

void OBDuplicate(CORBA::ServerRequest_ptr);
void OBRelease(CORBA::ServerRequest_ptr);

namespace CORBA
{

//
// The ServerRequest class
//
typedef OB::ObjVar< ServerRequest > ServerRequest_var;

class ServerRequest : public OB::RefCount
{
    Object_var object_;
    String_var name_;
    OB::Buffer& buf_;
    ULong offOut_;
    OB::InputStream in_;

    Context_var ctx_;
    NVList_var params_;
    Any_var result_;
    Any_var exception_;

    //
    // Unmarshal and marshal
    //
    void unmarshal();
    void unmarshalCtx();
    void marshal();

protected:

    ServerRequest(Object_ptr, const char*, OB::Buffer&, bool, ULong, ULong);

    // DynamicImpelmentations may create ServerRequests
    friend class ::PortableServer::DynamicImplementation;

public:

    static inline ServerRequest_ptr _duplicate(ServerRequest_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline ServerRequest_ptr _nil()
    { return 0; }

    OperationDef_ptr op_def();
    const char* op_name();
    Context_ptr ctx();
    void params(NVList_ptr);
    void result(Any*);
    void exception(Any*);
};

inline void
release(ServerRequest_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(ServerRequest_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA


namespace PortableServer
{

//
// The PortableServer::DynamicImplementation class
//
class DynamicImplementation : public virtual ServantBase
{
    //
    // Hide copy-constructor and assignment operator
    //
    DynamicImplementation(const DynamicImplementation&);
    void operator=(const DynamicImplementation&);

protected:

    DynamicImplementation() { }

public:

    //
    // Standard IDL to C++ Mapping
    //
    CORBA::Object_ptr _this();
    
    virtual void invoke(CORBA::ServerRequest_ptr request) = 0;
    
    virtual CORBA::RepositoryId
    _primary_interface(const ObjectId&, POA_ptr) = 0;

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    virtual OB::DispatchStatus _OB_dispatch(const char*, OB::Buffer&, bool,
					    CORBA::ULong, CORBA::ULong);
    
    virtual bool _OB_isDynamic();

    //
    // Create the OB::StubImplBase
    //
    virtual OB::StubImplBase* _OB_createStubImpl(POA_ptr, const ObjectId&,
                                                 const OB::OctetSeq&);
};

} // End of namespace PortableServer

#endif
